﻿using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.Axes;
using Arction.WPF.LightningChartUltimate.SeriesXY;
using Arction.WPF.LightningChartUltimate.Views.ViewXY;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace TestScrollingArction
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private const string licenseKey = "PUT YOUR LICENSE CODE HERE";
        private LightningChartUltimate m_chart;
        private Double m_dLatestTemperature;
        private Random m_random;
        private const int numSamples = 300000;
        private const int numSeries = 8;
        private DispatcherTimer dispatcherTimer;

        public MainWindow()
        {
            m_dLatestTemperature = 50.0;
            m_random = new Random((int)DateTime.Now.Ticks);
            dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 20);
            dispatcherTimer.Tick += dispatcherTimer_Tick;

            InitializeComponent();

            CreateChart();

            for (int i = 0; i < numSeries; i++)
            {
                CreateSeries(i);
            }
            

            FillSeries();

            SetClipAreas();
        }

        private void SetClipAreas()
        {
            List<ClipArea> clipAreasList = new List<ClipArea>();

            for (int i = 0; i < numSamples; i++)
            {
                if (i%100 == 0)
                {
                    ClipArea cp = new ClipArea((double)i, (double)(i + 30 > numSamples ? numSamples : i + 30), ClipDirection.X);
                    cp.MarginPixels = 1;
                    clipAreasList.Add(cp);
                }
            }

            for (int i = 0; i < numSeries; i++)
            {
                m_chart.ViewXY.PointLineSeries[i].SetClipAreas(clipAreasList.ToArray()); 
            }
        }

        void dispatcherTimer_Tick(object sender, EventArgs e)
        {
            //double time = 100 / dispatcherTimer.Interval.Milliseconds;
            double currentPosition = m_chart.ViewXY.LineSeriesCursors[0].ValueAtXAxis;
            double step = 10;

            if (m_chart != null)
            {
                m_chart.BeginUpdate();
                if (currentPosition + step > m_chart.ViewXY.XAxes[0].Maximum)
                {
                    m_chart.ViewXY.XAxes[0].ScrollPosition = m_chart.ViewXY.XAxes[0].Maximum + step;
                }

                m_chart.ViewXY.LineSeriesCursors[0].ValueAtXAxis = currentPosition + step;
                m_chart.EndUpdate();
            }
        }

        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            m_chart.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.None;
            dispatcherTimer.Stop();
        }

        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            m_chart.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.Scrolling;
            dispatcherTimer.Start();
        }

        private void FillSeries()
        {
            if (m_chart != null)
            {
                m_chart.BeginUpdate();
                for (int j = 0; j < numSeries; j++)
                {
                    for (int i = 0; i < numSamples; i++)
                    {
                        // An array for 1 point.
                        SeriesPoint[] points = new SeriesPoint[1];

                        //Store the X value
                        points[0].X = i;

                        if (i % 100 == 0)
                        {
                            points[0].Y = double.NaN;
                        }
                        else
                        {
                            //Randomize and store Y value 
                            points[0].Y = CalculateYValue();
                        }
                        //Add the new point into end of first PointLineSeries
                        m_chart.ViewXY.PointLineSeries[j].AddPoints(points, false);
                    } 
                }
                m_chart.EndUpdate();
            }
        }

        private void CreateSeries(int index)
        {
            if (m_chart != null)
            {
                m_chart.BeginUpdate();

                PointLineSeries series = new PointLineSeries(m_chart.ViewXY, m_chart.ViewXY.XAxes[0], m_chart.ViewXY.YAxes[0]);

                series.LineStyle.Color = DefaultColors.SeriesForBlackBackgroundWPF[index % DefaultColors.SeriesForBlackBackgroundWPF.Length];
                series.LineStyle.Width = 2f;
                series.LineStyle.AntiAliasing = LineAntialias.None;
                series.MouseInteraction = false;
                series.ScrollModePointsKeepLevel = 100;

                series.Title.Visible = true;
                series.Title.Font = new WPFFont(System.Drawing.FontFamily.GenericSansSerif, 10, System.Drawing.FontStyle.Regular);
                series.Title.Text = string.Empty;
                series.Title.HorizontalAlign = AlignmentHorizontal.Right;
                series.Title.VerticalAlign = AlignmentVertical.Top;
                series.Title.Offset = new PointIntXY(0, index * 25);
                series.Title.Fill.Style = RectFillStyle.ColorOnly;
                series.Title.Fill.Color = ChartTools.CalcGradient(System.Windows.Media.Color.FromArgb(200, 255, 255, 255), series.LineStyle.Color, 50);
                series.Title.Fill.GradientColor = ChartTools.CalcGradient(System.Windows.Media.Color.FromArgb(200, 0, 0, 0), series.LineStyle.Color, 50);
                series.Title.Color = Colors.White;
                series.Title.MouseInteraction = false;
                series.ShowInLegendBox = false;
                

                m_chart.ViewXY.PointLineSeries.Add(series);

                m_chart.EndUpdate();
            }
        }



        private void CreateChart()
        {
            // Clear any chartContainer's children.
            chartContainer.Children.Clear();

            if (m_chart != null)
            {
                // If a chart is already created, dispose it.
                m_chart.Dispose();
                m_chart = null;
            }

            // Create a new chart.
            m_chart = new LightningChartUltimate(licenseKey);
            m_chart.MouseDoubleClick += m_chart_MouseDoubleClick;
            m_chart.BeginUpdate();

            m_chart.Title.Visible = false;

            m_chart.ChartRenderOptions.MultiCoreProcessing = MultiCoreProcessing.Off;
            m_chart.ViewXY.ZoomPanOptions.AutoYFit.TargetAllYAxes = true;
            m_chart.ViewXY.ZoomPanOptions.MouseWheelZooming = MouseWheelZooming.Off;
            m_chart.ViewXY.ZoomPanOptions.RightToLeftZoomAction = RightToLeftZoomAction.Off;
            m_chart.ViewXY.ZoomPanOptions.LeftMouseButtonAction = MouseButtonAction.Zoom;
            m_chart.ViewXY.ZoomPanOptions.RightMouseButtonAction = MouseButtonAction.Pan;
            m_chart.ViewXY.ZoomPanOptions.PanDirection = PanDirection.Horizontal;
            m_chart.ViewXY.ZoomPanOptions.RectangleZoomDirectionLayered = RectangleZoomDirection.Horizontal;
            m_chart.ChartRenderOptions.AntiAliasLevel = 0;

            m_chart.ViewXY.Margins = new Thickness(80, 30, 10, 100);
            m_chart.ViewXY.AxisLayout.YAxesLayout = YAxesLayout.Stacked;
            m_chart.ViewXY.AxisLayout.YAxisAutoPlacement = YAxisAutoPlacement.Off;
            m_chart.ViewXY.AxisLayout.XAxisTitleAutoPlacement = false;
            m_chart.ViewXY.AxisLayout.YAxisTitleAutoPlacement = false;
            m_chart.ViewXY.AxisLayout.AxisGridStrips = XYAxisGridStrips.None;

            ViewXY v = m_chart.ViewXY;

            v.XAxes[0].ValueType = AxisValueType.Number;
            v.XAxes[0].SetRange(0, 4000);
            v.XAxes[0].ScrollingGap = 0;

            LineSeriesCursor cursor = new LineSeriesCursor(m_chart.ViewXY, m_chart.ViewXY.XAxes[0]);
            m_chart.ViewXY.LineSeriesCursors.Add(cursor);
            cursor.PositionChanged += cursor_PositionChanged;
            cursor.ValueAtXAxis = 0;
            cursor.LineStyle.Color = System.Windows.Media.Color.FromArgb(150, 255, 0, 0);
            cursor.SnapToPoints = true;
            cursor.TrackPoint.Color1 = Colors.White;

            v.YAxes.Clear();
            AxisY yAxis = new AxisY(v);

            yAxis.MajorGrid.Visible = false;
            yAxis.ValueType = AxisValueType.Number;
            yAxis.AllowAutoYFit = true;
            
            yAxis.Title.MouseInteraction = false;
            yAxis.Units.MouseInteraction = false;
            yAxis.SetRange(40.0, 100.0);
            v.YAxes.Add(yAxis);

            m_chart.ViewXY.LegendBox.Visible = false;

            ScrollBar sb = new ScrollBar(m_chart);
            sb.Offset = new PointIntXY(sb, 0, 35);
            sb.Scroll += sb_Scroll;
            sb.Maximum = numSamples;
            sb.Value = 0;
            sb.Minimum = sb.Value;
            sb.LargeChange = 4000;
            m_chart.ScrollBars.Add(sb);

            //Allow chart rendering
            m_chart.EndUpdate();

            chartContainer.Children.Add(m_chart);

        }

        void m_chart_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Left)
            {
                Point p = e.GetPosition(m_chart);

                double xValue;

                if (m_chart.ViewXY.XAxes[0].CoordToValue((int)p.X, out xValue, true))
                {
                    m_chart.ViewXY.LineSeriesCursors[0].ValueAtXAxis = xValue;
                }
            }
        }

        void cursor_PositionChanged(LineSeriesCursor sender, double newValue, ref bool cancelRendering)
        {
            double dSeriesYValue = 0;

            foreach (var currentSerie in m_chart.ViewXY.PointLineSeries)
            {
                bool bResolvedOK = SolveValueAccurate(currentSerie, newValue, out dSeriesYValue);

                if (bResolvedOK)
                {
                    currentSerie.Title.Visible = true;
                    currentSerie.Title.Text=dSeriesYValue.ToString();
                }
                else
                {
                    currentSerie.Title.Visible = false;
                }
            }
            

            m_chart.ScrollBars[0].Value = (ulong)newValue;

        }

        public bool SolveValueAccurate (PointLineSeries series, double xValue, out double yValue)
        {
            AxisY axisY = m_chart.ViewXY.YAxes[series.AssignYAxisIndex];
            yValue = 0;

            try
            {
                LineSeriesValueSolveResult lssvs = series.SolveYValueAtXValue(xValue);
                if (lssvs.SolveStatus == LineSeriesSolveStatus.OK)
                {
                    if (lssvs.YMax == -lssvs.YMin)
                    {
                        return false;
                    }
                    if (double.IsNaN(lssvs.YMax) || double.IsNaN(lssvs.YMin))
                    {
                        return false;
                    }
                    //PointLineSeries may have two or more points at same X value. If so, center it between min and max 
                    yValue = (lssvs.YMax + lssvs.YMin) / 2.0;
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception)
            {
                return false;
            }
        }

        void sb_Scroll(object sender, ScrollBarScrollType type, ulong oldValue, ulong newValue)
        {
            Double dMaximum = newValue + m_chart.ScrollBars[0].LargeChange;

            m_chart.ViewXY.XAxes[0].SetRange((int)newValue, (int)dMaximum);
        }

        private double CalculateYValue()
        {
            //Use latest value and generate some difference to it. 
            double dY = m_dLatestTemperature + (m_random.NextDouble() - 0.5) * 1.0;

            //Limit between 0 and 100
            if (dY > 100)
                dY = 100;
            if (dY < 40)
                dY = 40;

            m_dLatestTemperature = dY;

            return dY;
        }

        private void CheckBox_Click(object sender, RoutedEventArgs e)
        {
            m_chart.BeginUpdate();
            foreach (var series in m_chart.ViewXY.PointLineSeries)
            {
                series.ScrollingStabilizing = stabCheck.IsChecked.Value;
            }
            m_chart.EndUpdate();
            
        }
    }
}
